/** \file neck_function.h
*   \brief 首の動きに関するバインド用プログラム
*   \auther Shun.E
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "sample_player.h"

#include <rcsc/common/basic_client.h>
#include <rcsc/common/logger.h>
#include <rcsc/common/server_param.h>

#include "izk_main.h"
#include "neck_function.h"

#include <iostream>
#include <cstdlib> // exit
#include <cerrno> // errno
#include <cstring> // strerror
#include <csignal> // sigaction

#include "neck_offensive_intercept_neck.h"
#include "neck_goalie_turn_neck.h"

#include <rcsc/action/body_intercept.h>
#include <rcsc/action/neck_scan_field.h>
#include <rcsc/action/neck_turn_to_ball.h>
#include <rcsc/action/neck_turn_to_ball_or_scan.h>
#include <rcsc/action/neck_turn_to_point.h>
#include <rcsc/action/neck_turn_to_goalie_or_scan.h>
#include <rcsc/action/neck_turn_to_relative.h>

#include <rcsc/action/basic_actions.h>
#include <rcsc/action/view_synch.h>

#include <rcsc/player/player_agent.h>
#include <rcsc/player/debug_client.h>

using namespace luabind;
using namespace rcsc;

/** \fn l_Neck_OffensiveInterceptNeck()
*   \brief オフェンスがボールを追いかける際の首の動きを制御する.
*/
void
l_Neck_OffensiveInterceptNeck()
{
     // オフェンスの追跡行動
     izk_agent->setNeckAction( new Neck_OffensiveInterceptNeck() );
}

/** \fn l_Neck_GoalieTurnNeck()
*   \brief キーパーの首の動きを制御する.
*/
void
l_Neck_GoalieTurnNeck()
{
     // キーパーの首の動き
     izk_agent->setNeckAction( new Neck_GoalieTurnNeck() );
}

/** \fn l_Neck_ScanField()
*   \brief フィールドを見渡す.
*/
void
l_Neck_ScanField()
{
    // スキャンする
    izk_agent->setNeckAction(new Neck_ScanField());
}

/** \fn l_Neck_TurnToBall()
*   \brief ボールに首をむける. 
*/
void
l_Neck_TurnToBall()
{
    // スキャンする
    izk_agent->setNeckAction(new Neck_TurnToBall());
}

/** \fn l_Neck_TurnToBallOrScan()
*   \brief ボールに首をむける. 
*          ただし位置が未確定の場合はもう一度ボールを探す.
*/
void
l_Neck_TurnToBallOrScan()
{
    // スキャンする
    izk_agent->setNeckAction(new Neck_TurnToBallOrScan());
}

/** \fn l_Neck_TurnToGoalieOrScan()
*   \brief 敵のゴールキーパーに首をむける. 
*          ただし位置が未確定の場合はもう一度ゴールキーパーを探す.
*/
void
l_Neck_TurnToGoalieOrScan()
{
    // キーパーを探す
    izk_agent->setNeckAction(new Neck_TurnToGoalieOrScan());
}

/** \fn l_Neck_TurnToRelative(double degree)
*   \brief 体の向きから相対にdegreeの角度へ首を回転させる.
*   \param degree : 相対向き
*/
void
l_Neck_TurnToRelative(double degree)
{
    AngleDeg moddeg(degree);
    izk_agent->setNeckAction(new Neck_TurnToRelative( moddeg ));
}

/** \fn openFile
*   \brief Luaファイルを開く
*/
void
NeckFunction::openFile()
{

}


/** \fn bindFunc
*   \brief 関数のバインドを行う
*/
void
NeckFunction::bindFunc()
{
    // 関数やクラスを登録
    module(Luas)
    [
	def("Neck_OffensiveInterceptNeck", l_Neck_OffensiveInterceptNeck),
	def("Neck_GoalieTurnNeck", l_Neck_GoalieTurnNeck),
	def("Neck_ScanField", l_Neck_ScanField),
	def("Neck_TurnToBall", l_Neck_TurnToBall),
	def("Neck_TurnToBallOrScan", l_Neck_TurnToBallOrScan),
        def("Neck_TurnToGoalieOrScan",l_Neck_TurnToGoalieOrScan),		   
	def("Neck_TurnToRelative",l_Neck_TurnToRelative)
    ];
}
