--[[
   Strategy.lua
   戦術用プログラム
]]

--require("luaLib/FileOperation")
dofile("luaLib/FileOperation.lua")

fname = ""
local datacnt = 0
local disttable = {}
local angletable = {}
local dashtable = {}
local viewtable = {}
local tackleprob = 0.8
local dribdist = 13
local shootstep = 0

-- 戦術1専用変数
--[[local datacnt = 0
local bdist_table = {}
local drate_table = {}
local tprob_table = {} -- タックル成功推定用
local tdist_table = {} -- タックル向き許容範囲
local ddist_table = {} -- ドリブルの許容向き
local sstep_table = {}

-- ここがStrategyのメイン関数
function createStrategy(fname) --createStrategy
   --他の戦術が見つかれば, ここは書き換えてもよい.
   -- 今回はtokA1で開発したrobocup-editのデータを
   -- 読んで行動制御するように設定してある.
   if(readFile(fname)) then
     dataSet()
     closeFile()
     return true	--read robocup editor data
   end
   return false
end

-- 戦術1 : ファイルを読み込んで行動を制御する
function Strategy()
  datacnt = readData()  -- 繰り返し回数を取得
  for idx = 1, datacnt do
	local ball_dist = readData() -- ボール距離取得
	local dash_rate = readData() -- ダッシュ効率取得
	local tackle_prob = readData() -- タックル成功率取得
	local tackle_dist = readData() -- タックル範囲設定
	local dribble_dist = readData() -- ドリブル許容距離
	local shoot_step = readData() -- シュートステップを取得
	
	-- データ挿入
	table.insert(bdist_table, ball_dist)
	table.insert(drate_table, dash_rate)
	table.insert(tprob_table, tackle_prob)
	table.insert(tdist_table, tackle_dist)
	table.insert(ddist_table, dribble_dist)
	table.insert(sstep_table, shoot_step)
  end
  return 1
end


function editorData()
   local dist_min = 123456
   -- 行動を制御する
   for idx = 1, datacnt do
	if (bdist_table[idx] < dist_min
		and getBallDist() < bdist_table[idx]) then
		Dash(drate_table[idx])
		Tackle(tprob_table[idx], tdist_table[idx])
		Dribble(ddist_table[idx])
		Shoot(sstep_table[idx])
		
		-- 最小値更新
		dist_min = bdist_table[idx]
	end 
   end
   return 1
end]]

function createStrategy(fname) --createStrategy
   local filename = "Player/ActionParam/"..fname..".conf" 
   if(readFile(filename)) then
     dataSet()
     closeFile()
     return true	--read robocup editor data
   end
   return false
end

function dataSet()
   datacnt = readData()  -- 繰り返し回数を取得
   for i = 1, datacnt do
	local posX = readData()
	local posY = readData()
        local dist = readData()
        local angle = readData()
        local dashrate = readData()
        local viewtype = readData()	

	table.insert(disttable, tonumber(dist))
	table.insert(angletable, tonumber(angle))
	table.insert(dashtable, tonumber(dashrate))
	table.insert(viewtable, viewtype)
	--print(posX, posY)
   end
   tackleprob = tonumber(readData()) / 100.0
   dribdist = tonumber(readData())
   shootstep = tonumber(readData())
  return 1
end

-- 戦術2 : Strategy2
function Strategy2(fname)
   --繰り返し回数
   for i = 1, datacnt do
       -- ボールがすごく遠い位置にある
       if (disttable[i] >= 40.0 and getBallDist() >= 40.0) then
	   if (isBallAngle(angletable[i])) then
		Dash(dashtable[i])
		View(viewtable[i])
	   end	
	end
       --ボールが遠い位置にある	
       if (disttable[i] < 40.0 and
		 disttable[i] <= 20.0 and
		 getBallDist() < 40.0 and
		 getBallDist() <= 20.0) then
	   if (isBallAngle(angletable[i])) then
		Dash(dashtable[i])
		View(viewtable[i])
	   end	
	end	
       --ボールが普通の位置にある	
       if (disttable[i] < 20.0 and
		 disttable[i] <= 10.0 and
		 getBallDist() < 20.0 and
		 getBallDist() <= 10.0) then
	   if (isBallAngle(angletable[i])) then
		Dash(dashtable[i])
		View(viewtable[i])
	   end	
	end
       --ボールが近い位置にある	
       if (disttable[i] < 10.0 and
		 getBallDist() < 10.0) then
	   if (isBallAngle(angletable[i])) then
		Dash(dashtable[i])
		View(viewtable[i])
	   end	
	end
   end
   --print(tackleprob)
   Tackle(tackleprob, 80.0)
   Dribble(dribdist)
   Shoot(shootstep)
end

--該当するアングルの設定
function isBallAngle(angle)
    -- 右アングル
    local pangle = 0
    if (angle >= 0) then
       for i = 1, 6 do
	   local nangle = 30 * i
	   local bangle = getBallAngle() -- ボールの角度
	   if (bangle >= pangle 
		   and bangle < nangle) then
		   if (angle >= pangle 
			and angle < nangle) then
		   	--print(bangle)
			return true
		   end
	   end
	   pangle = nangle
        end
    end
    --左アングル
    if (angle < 0) then
       for i = 1, 6 do
	   local nangle = -30 * i
	   local bangle = getBallAngle() -- ボールの角度
	   if (bangle <= pangle 
		   and bangle > nangle) then
		   if (angle <= pangle 
			and angle > nangle) then
		   	--print(bangle..angle)
			return true
		   end
	   end
	   pangle = nangle
      end
   end

   return false
end
